@@ -26,10 +26,14 @@ struct dev_input;
 #define MAX_DEV_NAME	256
 
 struct device {
+	int id;
 	int fd;
 	void *data;
 	char name[MAX_DEV_NAME];
 	char path[PATH_MAX];
+	int type;
+	unsigned int usbid[2];	/* vendor:product for USB devices */
+	unsigned int flags;
 
 	int num_axes, num_buttons;
 	int *minval, *maxval;	/* input value range (default: -500, 500) */
@@ -42,7 +46,9 @@ struct device {
 	struct device *next;
 };
 
-int init_devices(void);
+void init_devices(void);
+void init_devices_serial(void);
+int init_devices_usb(void);
 
 void remove_device(struct device *dev);
 
