@@ -178,8 +178,21 @@ int open_dev_usb(struct device *dev)
 	/* set non-blocking */
 	fcntl(dev->fd, F_SETFL, fcntl(dev->fd, F_GETFL) | O_NONBLOCK);
 
-	if(cfg.led == 1 || (cfg.led == 2 && first_client())) {
+	if(cfg.led == LED_ON || (cfg.led == LED_AUTO && first_client())) {
 		set_led_evdev(dev, 1);
+	} else {
+		/* Some devices start with the LED enabled, make sure to turn it off
+		 * explicitly if necessary.
+		 *
+		 * XXX G.Ebner reports that some devices (SpaceMouse Compact at least)
+		 * fail to turn their LED off at startup if it's not turned explicitly
+		 * on first. We'll need to investigate further, but it doesn't seem to
+		 * cause any visible blinking, so let's leave the redundant call to
+		 * enable it first for now. See github pull request #39:
+		 * https://github.com/FreeSpacenav/spacenavd/pull/39
+		 */
+		set_led_evdev(dev, 1);
+		set_led_evdev(dev, 0);
 	}
 
 	/* fill the device function pointers */
@@ -291,9 +304,9 @@ static void set_led_evdev(struct device *dev, int state)
 }
 
 #define PROC_DEV	"/proc/bus/input/devices"
-struct usb_device_info *find_usb_devices(int (*match)(const struct usb_device_info*))
+struct usb_dev_info *find_usb_devices(int (*match)(const struct usb_dev_info*))
 {
-	struct usb_device_info *devlist = 0, devinfo;
+	struct usb_dev_info *devlist = 0, devinfo;
 	int i, buf_used, buf_len, bytes_read;
 	char buf[1024];
 	char *buf_pos, *section_start, *next_section = 0, *cur_line, *next_line;
@@ -336,7 +349,7 @@ struct usb_device_info *find_usb_devices(int (*match)(const struct usb_device_in
 				/* break to read from file again */
 				break;
 			}
-			/* set second newline to teminating null */
+			/* set second newline to terminating null */
 			next_section[1] = 0;
 			/* point to start of next section */
 			next_section += 2;
@@ -408,7 +421,7 @@ struct usb_device_info *find_usb_devices(int (*match)(const struct usb_device_in
 			 */
 			if(devinfo.num_devfiles > 0 && (!match || match(&devinfo))) {
 				/* add it to the list */
-				struct usb_device_info *node = malloc(sizeof *node);
+				struct usb_dev_info *node = malloc(sizeof *node);
 				if(node) {
 					if(verbose) {
 						logmsg(LOG_INFO, "found usb device [%x:%x]: \"%s\" (%s) \n", devinfo.vendorid, devinfo.productid,
@@ -473,7 +486,7 @@ alt_detect:
 		devinfo.num_devfiles = 1;
 
 		if(verbose) {
-			logmsg(LOG_INFO, "  trying \"%s\" ... ", devinfo.devfiles[0]);
+			logmsg(LOG_INFO, "  trying \"%s\" ... \n", devinfo.devfiles[0]);
 		}
 
 		if(stat(devinfo.devfiles[0], &st) == -1 || !S_ISCHR(st.st_mode)) {
@@ -502,7 +515,7 @@ alt_detect:
 		}
 
 		if(!match || match(&devinfo)) {
-			struct usb_device_info *node = malloc(sizeof *node);
+			struct usb_dev_info *node = malloc(sizeof *node);
 			if(node) {
 				if(verbose) {
 					logmsg(LOG_INFO, "found usb device [%x:%x]: \"%s\" (%s) \n", devinfo.vendorid, devinfo.productid,
@@ -528,4 +541,6 @@ alt_detect:
 	return devlist;
 }
 
+#else
+int spacenavd_dev_usb_linux_silence_empty_warning;
 #endif	/* __linux__ */
