@@ -97,7 +97,7 @@ int handle_hotplug(void)
 		logmsg(LOG_DEBUG, "\nhandle_hotplug called\n");
 	}
 
-	if(init_devices() == -1) {
+	if(init_devices_usb() == -1) {
 		return -1;
 	}
 	return 0;
@@ -145,4 +145,6 @@ static void poll_timeout(int sig)
 
 
 
+#else
+int spacenavd_hotplug_linux_shut_up_empty_source_warning;
 #endif	/* __linux__ */
