@@ -1,6 +1,6 @@
 /*
 spacenavd - a free software replacement driver for 6dof space-mice.
-Copyright (C) 2007-2019 John Tsiombikas <nuclear@member.fsf.org>
+Copyright (C) 2007-2021 John Tsiombikas <nuclear@member.fsf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -37,7 +37,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "proto_x11.h"
 #endif
 
+static void print_usage(const char *argv0);
 static void cleanup(void);
+static void redir_log(int fallback_syslog);
 static void daemonize(void);
 static int write_pid_file(void);
 static int find_running_daemon(void);
@@ -45,63 +47,85 @@ static void handle_events(fd_set *rset);
 static void sig_handler(int s);
 static char *fix_path(char *str);
 
-static char *cfgfile = DEF_CFGFILE;
+char *cfgfile = DEF_CFGFILE;
 static char *logfile = DEF_LOGFILE;
+static int pfd[2];
 
 int main(int argc, char **argv)
 {
 	int i, pid, ret, become_daemon = 1;
+	int force_logfile = 0;
 
 	for(i=1; i<argc; i++) {
-		if(argv[i][0] == '-' && argv[i][2] == 0) {
-			switch(argv[i][1]) {
-			case 'd':
-				become_daemon = !become_daemon;
-				break;
+		if(argv[i][0] == '-') {
+			if(argv[i][2] == 0) {
+				switch(argv[i][1]) {
+				case 'd':
+					become_daemon = !become_daemon;
+					break;
 
-			case 'c':
-				if(!argv[++i]) {
-					fprintf(stderr, "-c must be followed by the config file name\n");
-					return 1;
-				}
-				cfgfile = fix_path(argv[i]);
-				break;
-
-			case 'l':
-				if(!argv[++i]) {
-					fprintf(stderr, "-l must be followed by a logfile name or \"syslog\"\n");
-					return 1;
-				}
-				if(strcmp(argv[i], "syslog") == 0) {
-					logfile = 0;
-				} else {
-					logfile = fix_path(argv[i]);
-					if(strcmp(logfile, argv[i]) != 0) {
-						printf("logfile: %s\n", logfile);
+				case 'c':
+					if(!argv[++i]) {
+						fprintf(stderr, "-c must be followed by the config file name\n");
+						return 1;
 					}
+					cfgfile = fix_path(argv[i]);
+					break;
+
+				case 'l':
+					if(!argv[++i]) {
+						fprintf(stderr, "-l must be followed by a logfile name or \"syslog\"\n");
+						return 1;
+					}
+					if(strcmp(argv[i], "syslog") == 0) {
+						logfile = 0;
+					} else {
+						logfile = fix_path(argv[i]);
+						if(strcmp(logfile, argv[i]) != 0) {
+							printf("logfile: %s\n", logfile);
+						}
+						/* when the user specifies a log file in the command line
+						 * the expectation is to use it, regardless of whether
+						 * spacenavd is started daemonized or not.
+						 */
+						force_logfile = 1;
+					}
+					break;
+
+				case 'v':
+					verbose = 1;
+					break;
+
+				case 'V':
+					printf("spacenavd " VERSION "\n");
+					return 0;
+
+				case 'h':
+					print_usage(argv[0]);
+					return 0;
+
+				default:
+					fprintf(stderr, "invalid option: %s\n", argv[i]);
+					return 1;
 				}
-				break;
 
-			case 'v':
-				verbose = 1;
-				break;
-
-			case 'h':
-				printf("usage: %s [options]\n", argv[0]);
-				printf("options:\n");
-				printf(" -d: do not daemonize\n");
-				printf(" -c <file>: config file path (default: " DEF_CFGFILE ")\n");
-				printf(" -l <file>|syslog: log file path or log to syslog (default: " DEF_LOGFILE ")\n");
-				printf(" -v: verbose output\n");
-				printf(" -h: print usage information and exit\n");
+			} else if(strcmp(argv[i], "-version") == 0) {
+				printf("spacenavd " VERSION "\n");
 				return 0;
 
-			default:
-				fprintf(stderr, "unrecognized argument: %s\n", argv[i]);
+			} else if(strcmp(argv[i], "-help") == 0 || strcmp(argv[i], "--help") == 0) {
+				print_usage(argv[0]);
+				return 0;
+
+			} else {
+				fprintf(stderr, "invalid option: %s\n\n", argv[i]);
+				print_usage(argv[0]);
 				return 1;
 			}
+
 		} else {
-			fprintf(stderr, "unexpected argument: %s\n", argv[i]);
+			fprintf(stderr, "unexpected argument: %s\n\n", argv[i]);
+			print_usage(argv[0]);
 			return 1;
 		}
 	}
@@ -113,12 +137,18 @@ int main(int argc, char **argv)
 
 	if(become_daemon) {
 		daemonize();
+	} else {
+		if(force_logfile) {
+			redir_log(0);
+		}
 	}
 	write_pid_file();
 
 	logmsg(LOG_INFO, "Spacenav daemon " VERSION "\n");
 
 	read_cfg(cfgfile, &cfg);
+	prev_cfg = cfg;
+	pipe(pfd);
 
 	signal(SIGINT, sig_handler);
 	signal(SIGTERM, sig_handler);
@@ -126,6 +156,7 @@ int main(int argc, char **argv)
 	signal(SIGHUP, sig_handler);
 	signal(SIGUSR1, sig_handler);
 	signal(SIGUSR2, sig_handler);
+	signal(SIGPIPE, SIG_IGN);
 
 	init_devices();
 	init_hotplug();
@@ -186,6 +217,10 @@ int main(int argc, char **argv)
 		}
 #endif
 
+		/* also the self-pipe read-end for safe SIGHUP handling */
+		FD_SET(pfd[0], &rset);
+		if(pfd[0] > max_fd) max_fd = fd;
+
 		do {
 			/* if there is at least one device out of the deadzone and repeat is enabled
 			 * wait for only as long as specified in cfg.repeat_msec
@@ -224,6 +259,18 @@ int main(int argc, char **argv)
 	return 0;	/* unreachable */
 }
 
+static void print_usage(const char *argv0)
+{
+	printf("usage: %s [options]\n", argv0);
+	printf("options:\n");
+	printf(" -d: do not daemonize\n");
+	printf(" -c <file>: config file path (default: " DEF_CFGFILE ")\n");
+	printf(" -l <file>|syslog: log file path or log to syslog (default: " DEF_LOGFILE ")\n");
+	printf(" -v: verbose output\n");
+	printf(" -V,-version: print version number and exit\n");
+	printf(" -h,-help: print usage information and exit\n");
+}
+
 static void cleanup(void)
 {
 	struct device *dev;
@@ -245,30 +292,45 @@ static void cleanup(void)
 	remove(PIDFILE);
 }
 
-static void daemonize(void)
+static void redir_log(int fallback_syslog)
 {
-	int i, pid;
+	int i, fd = -1;
 
-	chdir("/");
-
-	/* redirect standard input/output/error
-	 * best effort attempt to make either the logfile or the syslog socket
-	 * accessible through stdout/stderr, just in case any printfs survived
-	 * the logmsg conversion.
-	 */
-	for(i=0; i<3; i++) {
-		close(i);
+	if(logfile) {
+		fd = start_logfile(logfile);
 	}
 
-	open("/dev/zero", O_RDONLY);
+	if(fd >= 0 || fallback_syslog) {
+		/* redirect standard input/output/error
+		 * best effort attempt to make either the logfile or the syslog socket
+		 * accessible through stdout/stderr, just in case any printfs survived
+		 * the logmsg conversion.
+		 */
+		for(i=0; i<3; i++) {
+			close(i);
+		}
 
-	if(!logfile || start_logfile(logfile) == -1) {
-		start_syslog(SYSLOG_ID);
+		open("/dev/zero", O_RDONLY);
+
+		if(fd == -1) {
+			fd = start_syslog(SYSLOG_ID);
+			dup(1);		/* not guaranteed to work */
+		} else {
+			dup(fd);
+		}
 	}
-	dup(1);
 
 	setvbuf(stdout, 0, _IOLBF, 0);
 	setvbuf(stderr, 0, _IONBF, 0);
+}
+
+static void daemonize(void)
+{
+	int pid;
+
+	chdir("/");
+
+	redir_log(1);
 
 	/* release controlling terminal */
 	if((pid = fork()) == -1) {
@@ -334,6 +396,15 @@ static void handle_events(fd_set *rset)
 	struct device *dev;
 	struct dev_input inp;
 
+	/* handle signal pipe */
+	if(FD_ISSET(pfd[0], rset)) {
+		int tmp;
+		read(pfd[0], &tmp, sizeof tmp);	/* eat up the junk char */
+
+		read_cfg(cfgfile, &cfg);
+		cfg_changed();
+	}
+
 	/* handle anything coming through the UNIX socket */
 	handle_uevents(rset);
 
@@ -356,6 +427,9 @@ static void handle_events(fd_set *rset)
 				/* ... and process it, possibly dispatching a spacenav event to clients */
 				process_input(dev, &inp);
 			}
+			/* flush any pending events if we run out of input */
+			inp.type = INP_FLUSH;
+			process_input(dev, &inp);
 		}
 		dev = next;
 	}
@@ -367,28 +441,48 @@ static void handle_events(fd_set *rset)
 	}
 }
 
+void cfg_changed(void)
+{
+	if(cfg.led != prev_cfg.led) {
+		struct device *dev = get_devices();
+		while(dev) {
+			if(is_device_valid(dev)) {
+				if(verbose) {
+					logmsg(LOG_INFO, "led %s, device: %s\n", cfg.led ? (cfg.led == LED_AUTO ? "auto" : "on"): "off", dev->name);
+				}
+				if(cfg.led == LED_ON || (cfg.led == LED_AUTO && first_client())) {
+					set_device_led(dev, 1);
+				} else {
+					set_device_led(dev, 0);
+				}
+			}
+			dev = dev->next;
+		}
+	}
+
+	if(strcmp(cfg.serial_dev, prev_cfg.serial_dev) != 0) {
+		struct device *dev, *iter = get_devices();
+		while(iter) {
+			dev = iter;
+			iter = iter->next;
+			if(strcmp(dev->path, prev_cfg.serial_dev) == 0) {
+				remove_device(dev);
+			}
+		}
+		init_devices_serial();
+	}
+
+	prev_cfg = cfg;
+}
+
 /* signals usr1 & usr2 are sent by the spnav_x11 script to start/stop the
  * daemon's connection to the X server.
  */
 static void sig_handler(int s)
 {
-	int prev_led = cfg.led;
-
 	switch(s) {
 	case SIGHUP:
-		read_cfg(cfgfile, &cfg);
-		if(cfg.led != prev_led) {
-			struct device *dev = get_devices();
-			while(dev) {
-				if(is_device_valid(dev)) {
-					if(verbose) {
-						logmsg(LOG_INFO, "turn led %s, device: %s\n", cfg.led ? "on": "off", dev->name);
-					}
-					set_device_led(dev, cfg.led);
-				}
-				dev = dev->next;
-			}
-		}
+		write(pfd[1], &s, 1);	/* write *something* to the pipe to trigger a re-read */
 		break;
 
 	case SIGSEGV:
